#include "library.h"

namespace py = pybind11;

void define_tech_tree(py::module & m)
{
    py::class_<TypeData>(m, "TypeData")
        .def_readonly("race",                 &TypeData::race)
        .def_readonly("mineral_cost",         &TypeData::mineralCost,                  "The mineral cost of the item.")
        .def_readonly("gas_cost",             &TypeData::gasCost,                      "The gas cost of the item.")
        .def_readonly("supply_cost",          &TypeData::supplyCost,                   "The supply cost of the item.")
        .def_readonly("build_time",           &TypeData::buildTime,                    "The build time of the item in seconds (should be 32 game updates per tick).")
        .def_readonly("is_unit",              &TypeData::isUnit,                       "Whether the item is a unit.")
        .def_readonly("is_building",          &TypeData::isBuilding,                   "Whether the item is a building.")
        .def_readonly("is_worker",            &TypeData::isWorker,                     "Whether the item is a worker.")
        .def_readonly("is_refinery",          &TypeData::isRefinery,                   "Whether the item is a refinery.")
        .def_readonly("is_supply_provider",   &TypeData::isSupplyProvider,             "Whether the item is a supply provider.")
        .def_readonly("is_resource_depot",    &TypeData::isResourceDepot,              "Whether the item is a resource depot.")
        .def_readonly("is_addon",             &TypeData::isAddon,                      "Whether the item is an add-on.")
        .def_readonly("build_ability",        &TypeData::buildAbility,                 "The ability that creates this item.")
        .def_readonly("warp_ability",         &TypeData::warpAbility,                  "The ability that creates this item via warp-in.")
        .def_readonly("what_builds",          &TypeData::whatBuilds,                   "The units that can build the item.")
        .def_readonly("required_units",       &TypeData::requiredUnits,                "Owning ONE of these (List[UnitType]) is required to make the item.")
        .def_readonly("required_upgrades",    &TypeData::requiredUpgrades,             "Having ALL of these (List[UPGRADE_ID]) is required to make the item.")
        .def_readonly("required_addons",      &TypeData::requiredAddons,               "A unit of this type (List[UnitType]) must be present next to the producer.")
        .doc() = R"(
            Allows you to get information about a unit type or upgrade. This is a read-only class.
        )";

    py::class_<TechTree>(m, "TechTree")
        .def("get_data", py::overload_cast<const UnitType &>(&TechTree::getData, py::const_))
        .def("get_data", py::overload_cast<const CCUpgrade &>(&TechTree::getData, py::const_), "Argument is either an instance of the class :class:`commandcenter.UnitType` or an instance of the class :class:`commandcenter.CCUpgrade`, depending on what information is wanted.")
		.def("suppress_warnings", &TechTree::setSuppressWarnings, "Suppress type and upgrade warnings." ,"b"_a)
		.doc() = R"(
            This class contains all information about units and what is required to
            build a certain unit and what builds it. It only has one method, which is
            used to look-up unit types properties.

            Instead of using TechTree, it's possible to use the functions in UnitType for
            structure, etc. In IDABot there is functions for getting data about upgrades.	
        )";
}
